@@ -8,7 +8,8 @@ from ...gpt_classes import ApiManager,InstructionManager,ResponseManager,PromptM
 from ...specializations import FileCollator,read_from_file_with_multiple_encodings
 from ...specializations.database_query import get_auto_db_query
 from ...specializations.responseContentParser import *
-from abstract_webtools import UrlManager, SafeRequest, url_grabber_component
+from abstract_webtools import urlManager, requestManager
+from abstract_webtools.url_grabber import url_grabber_component
 from abstract_gui import (
     get_event_key_js,
     text_to_key,
