@@ -57,13 +57,20 @@ class ResponseManager:
         self.prompt_as_previous_clone=None
     def get_response(self):
         # Convert the response to JSON directly within this method
-        self.current_response = safe_json_loads(self.response)
+        self.current_response = safe_json_loads(self.response) or {}
         self.current_created = self.current_response.get("created") or time.time()
         self.current_title = get_title(self.current_response) or get_any_value(self.current_response,'generate_title') or self.current_created or 'response'
         self.current_model = get_model(self.current_response) or get_any_value(self.current_response,'model') or self.model or 'default'
         self.current_usage = get_any_value(self.response,"usage")
         self.current_total_tokens = get_any_value(self.current_usage,"total_tokens")
         self.current_content = get_any_value(self.response,"content")
+        if not isinstance(self.current_content,dict):
+            try:
+                self.current_content = safe_json_loads(self.current_content)
+                self.current_response['choices'][0]['message']['content'] = self.current_content
+            except:
+                pass
+
         self.total_tokens_list.append({"time":self.current_created,"tokens":self.current_usage})
         self.get_all_instruction(self.current_content)
         self.query_js["response"]=self.current_response
