@@ -1,5 +1,7 @@
 import os
 import PySimpleGUI as sg
+from abstract_gui import *
+from .gui_utils import *
 from abstract_utilities import get_closest_match_from_list
 from abstract_utilities.list_utils import ensure_nested_list,find_original_case
 from abstract_utilities.class_utils import get_all_functions_for_instance,get_all_params,get_fun
@@ -37,14 +39,7 @@ class GuiUtilsManager:
             return sg.Frame(*args, **kwargs)
         # Add more PySimpleGUI component mappings as needed
 
-# Usage
-gui_utils = GuiUtilsManager()
 
-# For PySimpleGUI (Python):
-py_frame = [[gui_utils.make_component("Frame", title="PROGRESS", layout=[["InputText", "ProgressBar"]], output_language="python")
-]]
-# For JavaScript (HTML/JS):
-print(js_frame)
 
 def get_progress_frame()->list:
     return [
